const toml = require('toml')
const moment = require('moment')
const jsesc = require('jsesc')
const fs = require('fs')
const path = require('path')

const readPageWorker = (content_, obj, authors) => {
  let content = content_
  if (content === undefined) throw new Error('content is undefined')
  const headLength = content.indexOf('\n') + 1
  const head = content.substring(0, headLength)
  content = content.substring(headLength)
  const fmEnd = content.indexOf(head)
  if (fmEnd < 0) {
    console.log(jsesc(content))
    throw new Error(
      `Cannot find terminating ${head.substring(0, headLength - 1)}\n in file ${
        obj.filename
      }`,
    )
  }
  // eslint-disable-next-line no-param-reassign
  obj.content = content.substring(fmEnd + 4)
  try {
    // eslint-disable-next-line no-param-reassign
    obj.metadata = toml.parse(`${content.substring(0, fmEnd - 1)}\n`)
  } catch (e) {
    console.log(`${obj.filename}: Failed to parse metadata`)
    throw e
  }
  if (obj.metadata.published && obj.metadata.published) {
    // eslint-disable-next-line no-param-reassign
    obj.metadata.published = moment(obj.metadata.published)
  }
  if (obj.metadata.author) {
    const author = obj.metadata.author
    // eslint-disable-next-line no-param-reassign
    obj.metadata.author = authors.get(author) || { name: author }
  }
  return obj
}

module.exports = obj => {
  console.log('Build step: ParseHugo')
  const authors = new Map()
  JSON.parse(
    fs.readFileSync(
      path.join(obj.config.options.contentdir, 'authors.json'),
      'utf8',
    ),
  ).forEach(author => {
    authors.set(author.name, author)
  })
  return new Promise(resolve => {
    const pages = []
    obj.pages.forEach(page => {
      pages.push(readPageWorker(page.content, page, authors))
    })
    // eslint-disable-next-line no-param-reassign
    obj.pages = pages
    resolve(obj)
  })
}
