/*
 * This job reads files from config.articlesLocation and loads this list to `pages`
 */
const walk = require('walk')
const fs = require('fs')
const path = require('path')

module.exports = obj => {
  console.log('Build step: ReadFiles')
  const onFile = (root, fileStats, next, array) => {
    const filename = path.join(root, fileStats.name)
    const file = path.relative(obj.config.sourceDir, filename)
    fs.readFile(filename, 'utf8', (err, data) => {
      if (err) {
        console.log(`Error reading file ${filename}`)
        console.log(err)
      } else {
        array.push({
          filename,
          file,
          content: data,
        })
      }
      next()
    })
  }

  const promise1 = new Promise(resolve => {
    const pages = []
    walk
      .walk(path.join(obj.config.sourceDir, obj.config.articlesLocation))
      .on('file', (root, fileStats, next) => {
        onFile(root, fileStats, next, pages)
      })
      .on('errors', (root, nodeStatsArray, next) => {
        console.log('Walker error', root, nodeStatsArray)
        next()
      })
      .on('end', () => {
        resolve({ type: 'pages', val: pages })
      })
  })

  const promise2 = new Promise(resolve => {
    const redirects = []
    walk
      .walk(path.join(obj.config.sourceDir, obj.config.redirectsLocation))
      .on('file', (root, fileStats, next) => {
        onFile(root, fileStats, next, redirects)
      })
      .on('errors', (root, nodeStatsArray, next) => {
        console.log('Walker error', root, nodeStatsArray)
        next()
      })
      .on('end', () => {
        resolve({ type: 'redirects', val: redirects })
      })
  })
  return Promise.all([promise1, promise2]).then(arr => {
    arr.forEach(v => {
      // eslint-disable-next-line no-param-reassign
      obj[v.type] = v.val
    })
    return obj
  })
}
